package ucot.heuristic;


import java.util.Properties;
import java.util.logging.Logger;

import ucot.model.AnalyzeModel;
import ucot.model.ModelEditor;
import ucot.parser.ParsedData;
import ucot.parser.Sentence;
import ucot.parser.Word;
/**
 * Abbotts Heuristic is a way to etsimated which
 * parts of the speech are relevant and in which
 * way when doing object analyze.
 * 
 *
 */
public class AbbottsHeuristic implements HeuristicInterface {
	private Properties options = new Properties();
	
	private Logger logger = Logger.getLogger("ucot.parser");
	
	/** 
	 * @see ucot.heuristic.HeuristicInterface#doHeuristic(ucot.parser.ParsedData, ucot.model.AnalyzeModel)
	 */
    public void doHeuristic(ParsedData data, AnalyzeModel model) throws Exception {
		if (model == null || data == null) 
			return;

		// Signal that thing are starting to happend
		// so model observers can case updating things.
		model.updationStartedSignal(); 
		
		// Make data remember how it has been parsed.
        data.getUseCase().setHeuristic(this);
        
		ModelEditor editor = model.getEditor();
        
        // Loop through all of the sences.
        for (Sentence s : data.getSentences()) {
            logger.fine("Parsing " + s);
        	
        	
            // Loop through all the words.
            for (Word word : s.getWords()) {
                
                // Basic information from the word.
                String basicForm = word.getBasicForm();
                String wordClass = word.getWordClass();
                if (basicForm.length() == 0) continue;
                
                if (wordClass.equals("noun")) {
                    // Word is noun. So its is probably an entity.
                    if (! editor.containsEntity(basicForm)) editor.addEntity(basicForm);
                } else if (wordClass.equals("verb")) {
                    // Word is verb. So it is probably a operation
                    // between entities.
                    
                    // Check if there exist information 
                    // about the structure of the sentence. 
                    Word subject = word.getLinked("subject");
                    Word object = word.getLinked("object");
                    
                    if (null != subject && null != object) {
                        // There is structural information
                        // so we can create entity to entity
                        // relation.
                        
                    	if (! editor.containsEntity(subject.getBasicForm())) editor.addEntity(subject.getBasicForm());
                    	if (! editor.containsEntity(object.getBasicForm())) editor.addEntity(object.getBasicForm());
                        
                    	editor.addEntityInfluenceByMethod(subject.getBasicForm(), word.getBasicForm(), object.getBasicForm());
                    }
                    
                    if (null != subject){
                        // We know the subject
                        // so we can add "method" to the entity.
                        String entityName = subject.getBasicForm();
                        if (! editor.containsEntity(entityName)) editor.addEntity(entityName);

                        String methodName = word.getBasicForm();
                        if (! editor.containsMethod(entityName, methodName)) editor.addMethod(entityName, methodName);
                    }
                }
            }
        }
        model.readySignal();
    }

	
	/* (non-Javadoc)
	 * @see ucot.heuristic.HeuristicInterface#getName()
	 */
	public String getName() {		
		return "Abbots heuristic";
	}

    /* (non-Javadoc)
     * @see java.lang.Object#toString()
     */
    public String toString() {
        return this.getName();
    }
    
	/* (non-Javadoc)
	 * @see ucot.heuristic.HeuristicInterface#setOptions(java.util.Properties)
	 */
	public void setOptions(Properties options) {
		this.options = options;		
	}

	/* (non-Javadoc)
	 * @see ucot.heuristic.HeuristicInterface#getOptions()
	 */
	public Properties getOptions() {
		return options;
	}    
}
