package ucot.input;

import java.io.File;
import java.net.URL;
import java.util.Vector;
/**
 * Contains input handlers ({@link core.input.InputInterface}).
 *
 */
public class InputCollection {
	
	private Vector<InputInterface> inputs = new Vector<InputInterface>();
	/**
	 * Creates input collection.
	 */
	public InputCollection() {	
	}
	
	/**
	 * Adds given input handler to the collection.
	 * @param inf
	 */
	public void add(InputInterface inf){
		if (! inputs.contains(inf)) inputs.add(inf);
	}
	
	/**
	 * Returns input that can read file on given url
	 * @param url of the file we want to read
	 * @return InputInterface that can read file or null if none can
	 */
	public InputInterface getInputForUrl(URL url) {
		
		//Check that the file actually exists before trying InputAdapters
		File file = new File(url.getFile());
		if (!file.exists()) 
			return null;
		
		for(int i=0;i<getInputCount();i++) {
			if (getInput(i).canRead(url))
				return getInput(i);
		}		
		return null;
	}
	/**
	 * @return How many different handlers there are.
	 */
	public int getInputCount() {
		return inputs.size();
	}
	/**
	 * Returns handler by its index
	 * @param index Index of the wanted handler.
	 * @return The handler at that index.
	 */
	public InputInterface getInput(int index) {
		return inputs.get(index);
	}
}
