package ucot.input;

import java.io.IOException;
import java.net.URL;
import java.util.Properties;

/**
 * Interface for inputs used by UCOT core. Inputs are the modules that handle
 * reading data from given url and parsing usecases from it.
 * @author vevijopi
 *
 */
public interface InputInterface {
	/**
	 * Parses given usecase. Stores Parsed data to given ParsedData object.
	 * @param url Url where input is loaded from
	 * @return Steps read from the input
	 */
	public UseCaseCollection read(URL url) throws IOException;

	/**
	 * Test if this input can read file specified in url.
	 * @param url url of the file to try
	 * @return true if input can read the file
	 */
	public boolean canRead(URL url);
	
	/**
	 * Sets options for the adapter
	 * @param options options for the adapter
	 */
	public void setOptions(Properties options);
	
	/**
	 * Returns adapter's options
	 * @return adapter's options
	 */
	public Properties getOptions();

    /**
     * Returns inputs name as a String
     * @return String of inputs name
     */
    public String toString();
    
}


