package ucot.output;

import java.io.File;
import java.io.IOException;
import java.net.URI;
import java.net.URL;
import java.util.HashMap;
import java.util.Map;
import java.util.Properties;
import java.util.Set;

import net.sourceforge.gxl.GXLBool;
import net.sourceforge.gxl.GXLDocument;
import net.sourceforge.gxl.GXLEdge;
import net.sourceforge.gxl.GXLGraph;
import net.sourceforge.gxl.GXLNode;
import net.sourceforge.gxl.GXLString;
import net.sourceforge.gxl.GXLTup;
import ucot.heuristic.AbbottsHeuristic;
import ucot.input.SimpleInputAdapter;
import ucot.input.UseCaseCollection;
import ucot.model.AnalyzeModel;
import ucot.model.ModelEditor;
import ucot.parser.ParsedData;
import ucot.parser.SimpleInputParser;

public class GXLOutput implements OutputInterface {
	public static final String ATTRIBUTE_TYPE = "type";
	public static final String ATTRIBUTE_NAME = "name";
	public static final String ATTRIBUTE_TYPE_NAME = "attribute";
	public static final String METHOD_TYPE_NAME = "method";
	public static final String PARENT_TYPE_NAME = "parent";
	public void output(AnalyzeModel model, URL url) throws Exception {
		// Create document and elements
		GXLDocument gxlDocument = new GXLDocument();

		GXLGraph graph = new GXLGraph("Analyze model");
		Map<String, GXLNode> nameToNodeMap  = new HashMap<String,GXLNode>();
		
		ModelEditor editor = model.getEditor();
		
		Set<String> entitys = editor.getEntityNames();
		
		
		
		//URI nodeURI = new URI("http://www.gupro.de/GXL/gxl-1.0.gxl#NodeClass");
		//URI containsURI = new URI("http://www.gupro.de/GXL/gxl-1.0.gxl#contains");		
		// Create nodes
		for(String entity:entitys){
			GXLNode node = new GXLNode(entity);
			//node.setType(nodeURI);
			graph.add(node);
			node.setID(entity);
			nameToNodeMap.put(entity,node);
		}
		
		
		

		// Create method edges.
		for(String entity:entitys){
			Set<String> methods = editor.getMethodNames(entity);
			GXLNode node1 = nameToNodeMap.get(entity);
			for(String method : methods){
				Set<String> targets = editor.getEntitiesInfluencedByMethod(entity,method);
				for( String target : targets){
					GXLNode node2 = nameToNodeMap.get(target);
					GXLEdge edge1 = new GXLEdge(node1, node2);
					//edge1.setID(entity+"."+method);
					GXLString methodValue = new GXLString(METHOD_TYPE_NAME);
					edge1.setAttr(ATTRIBUTE_TYPE, methodValue);
					GXLString methodName = new GXLString(method);
					edge1.setAttr(ATTRIBUTE_NAME, methodName);
					edge1.setAttr("isdirected", new GXLBool(true));
					graph.add(edge1);
				}
			}
		}
		

		// Create attribute edges
		for(String entity:entitys){
			Set<String> attributes = editor.getAttributeNames(entity);
			GXLNode node1 = nameToNodeMap.get(entity);
			for(String attribute : attributes){
				GXLNode node2 = nameToNodeMap.get(attribute);
				GXLEdge edge1 = new GXLEdge(node1, node2);
				//edge1.setID(entity+"."+attribute);
				//edge1.setType(containsURI);
				GXLString attributeValue = new GXLString(ATTRIBUTE_TYPE_NAME);
				edge1.setAttr(ATTRIBUTE_TYPE, attributeValue);
				edge1.setAttr("isdirected", new GXLBool(true));
				GXLTup tup = new GXLTup();
				tup.add(new GXLString(editor.getAttributeFromCardinal(entity, attribute)));
				tup.add(new GXLString(editor.getAttributeToCardinal(entity, attribute)));
				edge1.setAttr("limits", tup);
				graph.add(edge1);
			}
		}
		
		
		// Create parent edges
		for(String entity:entitys){
			Set<String> parents = editor.getParents(entity);
			GXLNode node1 = nameToNodeMap.get(entity);
			for(String parent : parents){
				GXLNode node2 = nameToNodeMap.get(parent);
				GXLEdge edge1 = new GXLEdge(node1, node2);
				GXLString parentValue = new GXLString(PARENT_TYPE_NAME);
				edge1.setAttr(ATTRIBUTE_TYPE, parentValue);
				edge1.setAttr("isdirected", new GXLBool(true));
				graph.add(edge1);
			}
		}
		
		gxlDocument.getDocumentElement().add(graph);
//		Write the document to file
		try {
			File file = new File(url.getFile());
			gxlDocument.write(file);
		}
		catch (IOException ioe) {
			System.out.println("Error while writing to file: " + ioe);
		}
		
	}
	
	public String getName() {
		// TODO Auto-generated method stub
		return null;
	}
	
	public void setOptions(Properties options) {
		// TODO Auto-generated method stub
		
	}
	
	public Properties getOptions() {
		// TODO Auto-generated method stub
		return null;
	}
	
	/**
	 * Test program used while programming this class
	 * @param args
	 */
	public static void main(String args[]) throws Throwable{
		SimpleInputAdapter input = new SimpleInputAdapter();
		SimpleInputParser parser = new SimpleInputParser();
		AbbottsHeuristic heuristic = new AbbottsHeuristic();
		URL file = new URL("file", "", -1, "esimfile.txt");
		
		UseCaseCollection collection = input.read(file);
		ParsedData data = parser.parse(collection.getUseCase(0));
		AnalyzeModel model = null;
		heuristic.doHeuristic(data,model);
		
		
		GXLOutput output = new GXLOutput();
		try {
			output.output(null,null);
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
}
