package ucot.output;

import java.net.URL;
import java.util.Collections;
import java.util.HashSet;
import java.util.List;
import java.util.Properties;
import java.util.Set;
import java.util.Vector;

import ucot.model.AnalyzeModel;
import ucot.model.relation.Item;
import ucot.model.relation.Relation;
import ucot.model.relation.RelationType;
import ucot.model.relation.Thing;
/**
 * Simple outputer that prints AnalyzeModel to the
 * standard output.
 * @author pajumasu
 *
 */
public class Output implements OutputInterface {
	private Properties options;
    /**
     * Creates string representation for given thing.
     * Items are printed directly.
     * Relations are printed recursively so that both
     * to and from parts are queried again from getStringFor-method.
     * Mehod also queries for relations that originates from
     * the to part of handled relation and prints them too.
     * 
     * For example relations
     * [A -> B]
     * [B -> C]
     * are printed quite like
     * [A -> B
     *    [B -> C]
     * ]
     *
     * So printer relation is beign opened to the chain of relations
     * that are connected to it.
     * 
     * @param model AnalyzeModel which is used to get futher relations.
     * @param i the thing the string is wanted for.
     * @param preString String that is added before the representation (basically the indentation).
     * param handled Set of already handled things so we dont crash on loop relations.
     * @return
     */
    private String getStringFor(AnalyzeModel model, Thing i, String preString,
            Set<Thing> handled) {
        
        // The indentation that is added when we need to go to the next level
        // of indentation.
        String preAdd = "      ";
        
        // Create buffer for the string
        StringBuffer buffer = new StringBuffer();
        
        if (i instanceof Item) {
            
            // If i is simple item just print its name
            buffer.append(((Item) i).getName());
        
            
            
            // See if we have printed handled this item before
            // and dont print it sub relations if we have.
            if (! handled.contains(i)){
                handled.add(i);
                
                // Find out all the relations that
                // are originated from the to part
                // of this relation.
                /*Set<Relation> sub = model.getRelationsFrom(i);
                sub.removeAll(handled);
                if (sub.size() > 0) buffer.append("\n");
                
                // Build the strings for all the sub relations
                for (Relation relation : sub) {
                    buffer.append(getStringFor(model, relation, preAdd + preString, handled));
                    buffer.append("\n");
                } */ 
            }
        } else if (i instanceof Relation) {
            // Handle relation
            Relation r = (Relation) i;
            handled.add(r.getFrom());
            
            // Indent
            buffer.append(preString + "[");
            
            // Get the string for the from part by
            // recursive call.
            //buffer.append(getStringFor(model, r.getFrom(), "", handled));
            buffer.append(r.getFrom().getName());
            
            // The name of the relation.
            buffer.append(" --- " + r.getName() + " --> ");
            
            // Get the string for the to part by
            // recursive call.
            buffer.append(getStringFor(model, r.getTo(), preString, handled));
            if (buffer.lastIndexOf("\n")+1 == buffer.length()){
                buffer.append(preString+"]");
            } else {
                buffer.append("]");
            }
        
        }
        return buffer.toString();
    }
    
    
    public void output(AnalyzeModel model, URL url) {
        output(model, url,"");
    }
    
    public void output(AnalyzeModel model, URL url, String preString) {
        Set<String> entities = model.getEditor().getEntityNames();
        //Set<Relation> entityRelations = model.getEntityToEntityRelations();
        StringBuffer buffer = new StringBuffer();
        buffer.append("Entiteetit:\n");
        for (String ent : entities) {
            buffer.append(ent + "\n");
            Set<String> methods = model.getEditor().getMethodNames(ent);
            for(String method : methods){
                buffer.append("    ");
                buffer.append(method);
                buffer.append("\n");
            }
        }
        
        buffer.append("\n");
       /* buffer.append("Entiteettien vliset suhteet:\n");
        
        List<String> lines = new Vector<String>();
        
        for(Item entity : entities){
            Set<Relation> entityRelations = model.getRelationsFrom(entity);
            for (Relation rel : entityRelations) {
                if (rel.getType().equals(RelationType.INFLUENCE))
                    lines.add(getStringFor(model, rel, "", new HashSet<Thing>()));
            }
        }
        Collections.sort(lines);
        
        for (String line : lines) {
            buffer.append(line + "\n");
        }
        
        lines.clear();
        buffer.append("\n");
        */
        
        
        System.out.println(buffer.toString());
    }


	public String getName() {		
		return "Text output";
	}


	public void setOptions(Properties options) {
		this.options = options;		
	}


	public Properties getOptions() {
		return options;
	}
    
    
}
