package ucot.parser;
import java.util.Properties;

import ucot.input.UseCase;

/**
 * Interface for parsers within this program.
 * Parsers are given usecase and they return ParsedData
 * @author vevijopi
 *
 */
public interface ParserInterface {
	/**
	 * Parses given usecase and returns parsed data as a ParsedData object.
	 * @param usecase Usecase to parse
	 * @return parsed data as a ParsedData object.
	 */
	public ParsedData parse( UseCase useCase) throws Exception;
	
	/**
	 * Returns adapter's name
	 * @return
	 */
	public String getName();
	
	/**
	 * Sets options for the adapter
	 * @param options options for the adapter
	 */
	public void setOptions(Properties options);
	
	/**
	 * Returns adapter's options
	 * @return adapter's options
	 */
	public Properties getOptions();
}
