package ucot.test;

import junit.framework.TestCase;
import junit.framework.Assert;

import org.junit.Test;

import ucot.input.UseCaseCollection;
import ucot.parser.ParsedData;
import ucot.heuristic.AbbottsHeuristic;
import ucot.model.relation.RelationalAnalyzeModel;
import ucot.parser.SimpleInputParser;
import ucot.input.SimpleInputAdapter;
import java.net.URL;
import java.util.Set;
import ucot.model.relation.Item;
import ucot.model.relation.Relation;

/**
 * This class implements test for the heuristics.
 * @author tujupien
 */
public class HeuristicTest extends TestCase {
	
    final String testFileName = "Core-SimpleInputAdapter-UseCases-Test.txt";
    
    /**
     * This method tests Abbott's heuristic module and its initialization.
     * It is up to the tester to confirm that the heuristic gives correct
     * output.
     */
    @Test
	public void testAbbottsHeuristic() {
        System.out.println("TEST: Testing Abbott's Heuristic.");
        
        URL file;
        // Try to initialize test file URL.
        try {
            file = new URL("file", "", -1, testFileName);        
        } catch (Exception e) {
            System.out.println("TEST: Failed to initialize AbbottsHeuristic test. Stage 1.");
            Assert.fail();
            return;
        }
        // Initialize simple input adapter and load use cases.
        SimpleInputAdapter sia = new SimpleInputAdapter();
        UseCaseCollection collection;
        try {
            collection = sia.read(file);
        } catch (Exception e) {
            System.out.println("TEST: Failed to initialize AbbottsHeuristic test. Stage 2.");
            Assert.fail();
            return;
        }
        // Initialize simple input parser and parse data.
        SimpleInputParser sip = new SimpleInputParser();
        ParsedData data;
        try {
            data = sip.parse(collection.getUseCase(0));
        } catch (Exception e) {
            System.out.println("TEST: Failed to initialize AbbottsHeuristic test. Stage 3.");
            Assert.fail();
            return;
        }
        
        // Initialize heuristic module and do heuristic to the
        // parsed data.
        RelationalAnalyzeModel model = new RelationalAnalyzeModel();
        AbbottsHeuristic heuristic = new AbbottsHeuristic();
        try {
            heuristic.doHeuristic(data, model);
        } catch (Exception e) {
            System.out.println("TEST: AbbottsHeuristic failed.");
            System.out.println("TEST: -- " + e.getMessage());
            Assert.fail();
            return;
        }
        
        Set<Item> ent = model.getItemsType("entity");
        // Print entities from heuristic module.
        System.out.println("TEST: Abbott's heuristic found the following entities and their methods:");
        for ( Item x: ent ) {
            System.out.println("TEST: -- " + x.getName() + " [" + x.getTypeName() + "]");
            Set<Relation> rel = model.getRelationsFrom(x);
            for ( Relation y: rel ) {
                //if ( y.getName().equals("contains") ) continue;
                System.out.println("TEST: ---- " + y.getName() + " -> " + y.getTo().getName());
            }
        }
        
        // It is up to the tester to validate test data.
		Assert.assertTrue(true);
	}

}
