package ucot.test;

import java.util.Set;

import junit.framework.Assert;

import org.junit.Test;

import ucot.model.ModelEditor;
import ucot.model.relation.RelationalAnalyzeModel;

public class RelationalModelTest {
	@Test
	public void testSimpleParentChildRelation(){
		RelationalAnalyzeModel am = new RelationalAnalyzeModel();
		ModelEditor me = am.getEditor();
		me.addEntity("Auto");
		me.addEntity("Rekka-auto");
		me.addParent("Rekka-auto","Auto");
		Set<String> parents = me.getParents("Rekka-Auto");
		Set<String> children = me.getChildren("Auto");
		Assert.assertEquals(1, children.size());
		Assert.assertEquals(1, parents.size());
		Assert.assertEquals(true, parents.contains("Auto"));
		Assert.assertEquals(true, children.contains("Rekka-auto"));
		
	}
	
	@Test
	public void testComplexParentChildRelation(){
		RelationalAnalyzeModel am = new RelationalAnalyzeModel();
		ModelEditor me = am.getEditor();
		me.addEntity("Auto");
		me.addEntity("Laiva");
		me.addEntity("Asunto");
		me.addEntity("Asuntoauto");
		me.addEntity("Asuntolaiva");
		me.addParent("Asuntoauto","Asunto");
		me.addParent("Asuntoauto","Auto");
		me.addParent("Asuntolaiva","Asunto");
		me.addParent("Asuntolaiva","Laiva");
		Set<String> parents = me.getParents("Asuntoauto");
		Set<String> children = me.getChildren("Asunto");
		Assert.assertEquals(2, children.size());
		Assert.assertEquals(2, parents.size());
		Assert.assertEquals(true, parents.contains("Auto"));
		Assert.assertEquals(true, parents.contains("Asunto"));
		Assert.assertEquals(true, children.contains("Asuntolaiva"));
		Assert.assertEquals(true, children.contains("Asuntoauto"));
		
	}

	@Test
	public void testRemovalsParentChildRelation(){
		RelationalAnalyzeModel am = new RelationalAnalyzeModel();
		ModelEditor me = am.getEditor();
		me.addEntity("Auto");
		me.addEntity("Laiva");
		me.addEntity("Asunto");
		me.addEntity("Asuntoauto");
		me.addEntity("Asuntolaiva");
		me.addParent("Asuntoauto","Asunto");
		me.addParent("Asuntoauto","Auto");
		me.addParent("Asuntolaiva","Asunto");
		me.addParent("Asuntolaiva","Laiva");
		
		me.removeChild("Asunto","Asuntolaiva");
		
		Set<String> parents = me.getParents("Asuntoauto");
		Set<String> children = me.getChildren("Asunto");
		Assert.assertEquals(1, children.size());
		Assert.assertEquals(2, parents.size());
		Assert.assertEquals(true, parents.contains("Auto"));
		Assert.assertEquals(true, parents.contains("Asunto"));
		Assert.assertEquals(true, children.contains("Asuntoauto"));
		
		me.removeAllParents("Asuntoauto");
		
		parents = me.getParents("Asuntoauto");
		children = me.getChildren("Asunto");
		Assert.assertEquals(0, children.size());
		Assert.assertEquals(0, parents.size());
	}
	
}
