package ucot.test;

import junit.framework.Assert;

import org.junit.Test;

import ucot.input.UseCaseCollection;
import ucot.input.UseCase;

/**
 * This class implements tests for use cases.
 * @author tujupien
 */
public class UseCaseTest {
	UseCase useCase = new UseCase();
    UseCaseCollection collection = new UseCaseCollection();
	
    /**
     * Method for testing use case object's name attribute.
     */
	@Test
	public void testName(){
		System.out.println("TEST: Testing UseCase name attribute.");
		String str = "abCD12!'!\"#";
		useCase.setName(str);
		Assert.assertEquals(str,useCase.getName());
	}
	
	/**
	 * Method for testing use case object's step attribute.
	 */
	@Test
	public void testStep() {
		System.out.println("TEST: Testing UseCase steps.");
		String str = "abcABC,%/,94584,\"'~d";
		useCase.addStep(str,null);
		Assert.assertEquals(str,useCase.getStep(useCase.getStepCount()-1).getStep());
	}
    
	/**
	 * Testing use case collection's merge operation. Use cases should
	 * equal each other exactly when they have the same name and exactly
	 * same steps.
	 */
    @Test
    public void testUseCaseMerge() {
        UseCase uc[] = new UseCase[4];
        // Generate test use cases.
        for ( int i = 0; i < uc.length; i++ ) uc[i] = new UseCase();
        uc[0].setName("Useless Case");
        uc[1].setName("Case");
        uc[1].addStep("Stepping around the floor.",null);
        uc[2].setName("Case Use");
        uc[3].setName("Case");
        uc[3].addStep("Dancing around the world.",null);
        
        // Create two collections with overlapping use cases.
        collection.clear();
        UseCaseCollection anotherCollection = new UseCaseCollection();
        anotherCollection.addUseCase(uc[0]);
        anotherCollection.addUseCase(uc[1]);
        anotherCollection.addUseCase(uc[2]);
        collection.addUseCase(uc[2]);
        collection.addUseCase(uc[3]);
        
        // Try merging these collections.
        collection.merge(anotherCollection);
        Assert.assertEquals(collection.getUseCaseCount(), 4);
    }
    
    /**
     * Test use case existance checking method.
     */
    @Test
    public void testUseCaseExists() {
        UseCase uc[] = new UseCase[5];
        // Generate test data.
        for ( int i = 0; i < uc.length; i++ ) uc[i] = new UseCase();
        uc[0].setName("Useless Case");
        uc[1].setName("Case");
        uc[1].addStep("Stepping around the floor.",null);
        uc[2].setName("Case Use");
        uc[3].setName("Case");
        uc[3].addStep("Dancing around the world.",null);
        uc[4].setName("Case");
        uc[4].addStep("Flooring around the dance.",null);
        
        collection.clear();
        for ( int i = 0; i <= 3; i++ ) collection.addUseCase(uc[i]);
        
        // Use case index 3 should exist and 4 not.
        Assert.assertTrue(collection.exists(uc[3]));
        Assert.assertFalse(collection.exists(uc[4]));
    }
}
