package ucot.ui;

import java.util.Set;

import ucot.core.ControlInterface;

/**
 * Interface for the UI. A means for the core component to notify UI
 * that certain actions have been taken. For example file has been loaded
 * @author vevijopi
 *
 */
public interface UIInterface {
	/**
	 * Core signals user interface that analyze model has been exported
	 *
	 */
	public void exportDone();
	
	/**
	 * Core signals user interface that analyzemodel has been loaded
	 *
	 */
	public void analyzeModelLoaded();
	
	/**
	 * Core signals user interface that usecases have been loaded from file
	 *
	 */
	public void useCasesLoaded();
	
	/**
	 * Core signals user interface that usecases have been parsed,
	 * ran heuristic on and been added to given analyze model
	 */
	public void useCaseAdded();
	
	/**
	 * Set control interface
	 * @param a control interface (UCOT core)
	 */
	public void setControlInterface(ControlInterface a);
	
	/**
	 * Prints error message to the user.
	 * @param errorMessage Description of the error.
	 * @param errorTitle title of the dialog
	 */
	public void printError(String errorMessage, String errorTitle);
	public void printError(String errorMessage);
	
	/**
	 * Shows user a dialog asking a question
	 * @param title title of the question dialog 
	 * @param question question to ask
	 * @param options options for the dialog
	 * @return User's choice (e.g. button)
	 */
	public Object questionDialog(String title, String question, int options);
	public Object questionDialog(String title, String question);
	public Object questionDialog(String question);
	
	
	
}
