package ucot.ui.gui;

import java.awt.Dimension;
import java.awt.Graphics;

import javax.swing.JPanel;

import ucot.input.UseCase;
import ucot.ui.UseCasePanelInterface;

/**
 * Simple panel for showing usecase steps
 * 
 * @author ilanliuk
 *
 */
public class SimpleUseCasePanel extends JPanel implements UseCasePanelInterface {
	
	private UseCase usecase;
	
	public SimpleUseCasePanel() {
		super();
	}
	
	
	/**
	 * Sets the usecase for display.
	 */
	public void showUseCase(UseCase usecase) {
		
		this.usecase = usecase;        
		this.repaint();
		
	}
	
	/**
	 * clears usecase from panel.
	 */
	public void Clear() {
		this.usecase = null;
		this.repaint();
	}
	
	@Override
	protected void paintComponent(Graphics g) {
		super.paintComponent(g);        
		
		if (usecase != null) {
			
			
			int x = 5;
			int y = (g.getFontMetrics().getHeight() + 5);
			int w = 0;
			int h = 10;
			
			g.drawString(usecase.getName(), x ,y);
			h += g.getFontMetrics().getHeight() + 5;
			
			int dx = (int) g.getFontMetrics().getStringBounds(usecase.getName(), g).getWidth();
			g.drawLine(x - 3, y + 5, dx + 6, y + 5);
			w = dx + 10;
			
			
			for (int i = 0; i < usecase.getStepCount(); i++) {
				h += g.getFontMetrics().getHeight() + 5;
				g.drawString((i + 1) + ". " + usecase.getStep(i).getStep(),x ,y * (i + 2) + 5);
				dx = (int) g.getFontMetrics().getStringBounds((i+1)+". "+
						usecase.getStep(i).getStep(),g).getWidth();
				if ( dx + 10 > w ) w = dx + 10;
			}
			
			if (usecase.isUseCaseAnalyzed()) {
				y = y * (usecase.getStepCount()+3) + 5;
				
				//h += 2* (g.getFontMetrics().getHeight() + 5);
				g.drawString("Parser: " + usecase.getParserName(), x,y);
				g.drawString("Heuristic: " + usecase.getHeuristicName(), x,y + g.getFontMetrics().getHeight() + 5);
				h = y + g.getFontMetrics().getHeight() + 10;
			}
			
			this.setPreferredSize(new Dimension(w, h));
		} else {
			this.setPreferredSize(new Dimension(0, 0));
		}
		this.revalidate();
		
	}
	
	
	
}
