package ucot.ui.gui.tree;

import java.awt.Point;
import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.awt.dnd.DnDConstants;
import java.awt.dnd.DragGestureEvent;
import java.awt.dnd.DragGestureListener;
import java.awt.dnd.DragSource;
import java.awt.dnd.DropTarget;
import java.awt.dnd.DropTargetDragEvent;
import java.awt.dnd.DropTargetDropEvent;
import java.awt.dnd.DropTargetEvent;
import java.awt.dnd.DropTargetListener;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.List;
import java.util.Set;

import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;
import javax.swing.JTree;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.tree.TreeNode;
import javax.swing.tree.TreePath;
import javax.swing.tree.TreeSelectionModel;

import ucot.core.ControlInterface;
import ucot.model.AnalyzeModel;
import ucot.model.relation.Entity;
import ucot.model.relation.Method;
import ucot.model.relation.RelationalAnalyzeModel;
import ucot.ui.gui.DotPanel;
import ucot.ui.gui.EntityPropertiesDialog;

public class AnalyzeModelTree extends JTree implements DragGestureListener, DropTargetListener {
	
	// TreeSelectionListener for reacting when user selects node
	private TreeSelectionListener treeSelectionListener;
	
	// who is notified when user selects a node..
	private DotPanel panel = null;
	
	private DragSource dragSource;
	private DropTargetListener dtListener;
	private DropTarget dropTarget;
	private AnalyzeTreeModel atm;
	private AnalyzeModel analyzemodel = null;
	
	public void createMouseListener(final ControlInterface core, final EntityPropertiesDialog entityPropertiesDialog) {
		addMouseListener(new MouseListener() {
			
			public void mouseClicked(MouseEvent e) {
				// See if we cliked to any node in the tree.
				final TreePath selectionPath = getPathForLocation(e.getX(), e.getY());
				if (selectionPath != null){
					final TreeItem selected = (TreeItem) selectionPath.getLastPathComponent();
					
					
					// If user clicked second mouse button
					// (MouseEvent.BUTTON2 is scroll)
					if (e.getButton() == MouseEvent.BUTTON3) {
						
						System.out.println("Tree: *RIGHT CLICK*");
						
						// Set selection right.
						setSelectionPath(selectionPath);
						
						// Creating popup if something is selected.
						if (selectionPath != null) {
							JPopupMenu menu = new JPopupMenu();
							
							// Choices:
							JMenuItem createEntity = new JMenuItem("New entity..");
							JMenuItem modify = new JMenuItem("Modify..");
							JMenuItem deleteEntity = new JMenuItem("Delete entity");
							JMenuItem deleteRelation = new JMenuItem("Remove method target");
							JMenuItem removeAttribute = new JMenuItem("Remove from attributes");
							JMenuItem removeParent = new JMenuItem("Remove Parent");
							
							// Add listener for createEntity event.
							createEntity.addActionListener(new ActionListener() {
								
								public void actionPerformed(ActionEvent e) {
									System.out.println("Popup createEntity");
									
									// Add entity called "new entity" to
									// analyzeModel and
									// then open modifying dialog with it..
									
									// core.getAnalyzeModel().addItem(ent);
									entityPropertiesDialog.modifyEntityProperties("new entity", core.getAnalyzeModel());
								}
								
							});
							
							// Add listener for modify event.
							modify.addActionListener(new ActionListener() {
								public void actionPerformed(ActionEvent e) {
									System.out.println("Popup modify: "
											+ selected.toString());
									// entProperties.setVisible(true);
									if (selected instanceof EntityTreeItem) {
										entityPropertiesDialog.modifyEntityProperties(selected.getName(), core.getAnalyzeModel());
									}
								}
							});
							
							// if selected item is AttributeTreeItem add listener 
							// for remove attribute event
							if (selected instanceof AttributeTreeItem) {
								removeAttribute.addActionListener(new ActionListener() {
									public void actionPerformed(ActionEvent e) {
										AttributeTreeItem item = (AttributeTreeItem) selected;
										String entity = item.getParent().getParent().getName();
										core.getAnalyzeModel().getEditor().removeAttribute(entity, item.getName());
									}
								});
								menu.add(removeAttribute);
							}
							
							// if selected item is InheritanceTreeItem add
							// listener for remove parent event
							if (selected instanceof InheritanceTreeItem) {
								removeParent.addActionListener(new ActionListener() {
									public void actionPerformed(ActionEvent e) {
										InheritanceTreeItem parent = (InheritanceTreeItem) selected;
										String entity = parent.getParent().getParent().getName();
										core.getAnalyzeModel().getEditor().removeParent(entity, parent.getName());
									}
								});
								menu.add(removeParent);
							}
							
							if (selected instanceof EntityTreeItem
									&& selected.getParent() instanceof EntitiesTreeItem) {
								// Add listener for delete event.
								deleteEntity.addActionListener(new ActionListener() {
									public void actionPerformed(ActionEvent e) {
										System.out.println("Popup delete");
										if (null != selected && (selected instanceof EntityTreeItem)) {
											core.getAnalyzeModel().getEditor()
											.removeEntity(selected.getName());
										}
									}
								});
								menu.add(deleteEntity);
							}
							
							if (selected instanceof EntityTreeItem
									&& selected.getParent() instanceof MethodTreeItem) {
								// Add listener for delete event.
								deleteRelation.addActionListener(new ActionListener(){
									public void actionPerformed(ActionEvent e) {
										System.out.println("Popup delete relation");
										String fromEntity = selected.getParent().getParent().getParent().getName();
										String method = selected.getParent().getName();
										core.getAnalyzeModel().getEditor().removeEntityInfluenceByMethod(fromEntity, method, selected.getName());
									}
								});
								menu.add(deleteRelation);
							}
							
							// Add choices to menu.
							if (selected instanceof EntitiesTreeItem) {
								menu.add(createEntity);
							}
							if (selected instanceof EntityTreeItem && 
									selected.getParent() instanceof EntitiesTreeItem) {
								menu.add(modify);
							}
							// Show popup menu relative to the components
							// coordinates.
							// This shows popup exactly where user pressed.
							menu.show(e.getComponent(), e.getX(), e.getY());
						}
					}
				}
			}
			public void mouseEntered(MouseEvent e) {}
			public void mouseExited(MouseEvent e) {}
			public void mousePressed(MouseEvent e) {}
			public void mouseReleased(MouseEvent e) {}
		});
	}
	
	public AnalyzeModelTree(AnalyzeModel model) {
		
		//this.setDragEnabled(true);
		this.dragSource = DragSource.getDefaultDragSource();
		this.getSelectionModel().setSelectionMode(TreeSelectionModel.SINGLE_TREE_SELECTION);
		
		
		this.dragSource.createDefaultDragGestureRecognizer(
				this, DnDConstants.ACTION_MOVE, this );
		
		atm = new AnalyzeTreeModel(model);
		
		this.setModel(atm);
		this.dtListener = this;
		
		// component, ops, listener, accepting
		this.dropTarget = new DropTarget(this,
				this.acceptableActions,
				this.dtListener,
				true);
		
		// create the TreeSelectionListener and override selection event..
		treeSelectionListener = new TreeSelectionListener() {
			
			public void valueChanged(TreeSelectionEvent e) {
				
				TreeItem item = (TreeItem) e.getPath().getLastPathComponent();
				
				// if dotpanel is assigned..
				if (panel == null) return;
				if (item == null) return;
				
				//Object data = item.getData();
				// if the node selected is not an itemNode there is no reason to continue..
				//if (! ( data instanceof Entity )) return;
				
				
				// if selected node is entity, notify the structurePanel 
				// that selectection has changed
				
				//if ( ((Entity) data).getTypeName().equals("entity"));
				//structurePanel.updateModel(null, (Entity) data);				
				
				//if ( ((Entity) data).getTypeName().equals("entity"))
				
				//panel.updateModel(atm.getAnalyzeModel());
				
				RelationalAnalyzeModel highlightModel = new RelationalAnalyzeModel();
				if ( item instanceof EntityTreeItem ) {
					highlightModel.getEditor().addEntity(item.getName());
				}
				if ( item instanceof MethodTreeItem ) {
					highlightModel.getEditor().addEntity(item.parent.parent.getName());
					Set<String> targets = atm.getAnalyzeModel().getEditor().
					getEntitiesInfluencedByMethod(item.parent.parent.getName(), item.getName());
					highlightModel.getEditor().addMethod(item.parent.parent.getName(), item.getName());
					for ( String target: targets ) {
						highlightModel.getEditor().addEntity(target);
						highlightModel.getEditor().addEntityInfluenceByMethod(
								item.parent.parent.getName(), item.getName(), target);
					}
				}
				if ( item instanceof MethodsTreeItem ) {
					highlightModel.getEditor().addEntity(item.parent.getName());
					List<TreeItem> children = item.getChildren();
					for ( TreeItem child: children ) {
						Set<String> targets = atm.getAnalyzeModel().getEditor().
						getEntitiesInfluencedByMethod(item.parent.getName(), child.getName());
						highlightModel.getEditor().addMethod(item.parent.getName(), child.getName());
						for ( String target: targets ) {
							highlightModel.getEditor().addEntity(target);
							highlightModel.getEditor().addEntityInfluenceByMethod(
									item.parent.getName(), child.getName(), target);
						}
					}
				}
				if ( item instanceof AttributeTreeItem ) {
					highlightModel.getEditor().addEntity(item.parent.parent.getName());
					highlightModel.getEditor().addEntity(item.getName());
					highlightModel.getEditor().addAttribute(item.parent.parent.getName(), item.getName());
				}
				if ( item instanceof AttributesTreeItem ) {
					highlightModel.getEditor().addEntity(item.parent.getName());
					List<TreeItem> children = item.getChildren();
					for ( TreeItem child: children ) {
						highlightModel.getEditor().addEntity(child.getName());
						highlightModel.getEditor().addAttribute(item.parent.getName(), child.getName());
					}
				}
				if ( item instanceof InheritanceTreeItem ) {
					highlightModel.getEditor().addEntity(item.parent.parent.getName());
					highlightModel.getEditor().addEntity(item.getName());
					highlightModel.getEditor().addParent(item.parent.parent.getName(), item.getName());
				}
				if ( item instanceof InheritancesTreeItem ) {
					highlightModel.getEditor().addEntity(item.parent.getName());
					List<TreeItem> children = item.getChildren();
					for ( TreeItem child: children ) {
						highlightModel.getEditor().addEntity(child.getName());
						highlightModel.getEditor().addParent(item.parent.getName(), child.getName());
					}
				}
				
				panel.highlight(highlightModel);
			}
			
		};
		this.addTreeSelectionListener( treeSelectionListener );
		
	}
	
	private int acceptableActions = DnDConstants.ACTION_COPY;
	
	public void dragGestureRecognized(DragGestureEvent dragEvent) {
		TreePath path = getSelectionModel().getSelectionPath();
		if (null != path){
			TreeItem treeItem = (TreeItem) path.getLastPathComponent();
			/*Object data = treeItem.getData();
			 if (data instanceof Entity){
			 dragEvent.startDrag(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR), 
			 new DataTransfer(data));
			 }*/
			
		}
		
	}
	
	public void dragEnter(DropTargetDragEvent arg0) {
		System.out.println(arg0);
	}
	
	public void dragExit(DropTargetEvent arg0) {
		System.out.println(arg0);
		
	}
	
	public void dragOver(DropTargetDragEvent arg0) {
		
		
	}
	
	public void drop(DropTargetDropEvent dtde){
		
		System.out.println("DROP");
		//The Drag operation has terminated with a Drop on this DropTarget
		Transferable transferable = dtde.getTransferable();
		System.out.println(transferable);
		
		try{
			
			Entity entity = (Entity) transferable.getTransferData(DataFlavor.stringFlavor);			
			
			
			Point loc = dtde.getLocation();
			
			TreePath destinationPath = getPathForLocation(loc.x, loc.y);
			if (destinationPath != null) {
				Entity parentEntity = getEntityFromPath(destinationPath);
				Method relationMethod = getMethodFromPath(destinationPath);
				
				if (parentEntity != null && relationMethod != null){
					
					//analyzemodel.addRelation(new Relation(parentEntity,relationMethod.getName(),entity));
					
					dtde.dropComplete(true);
				}
			}
		}catch(Exception e){
			e.printStackTrace();
		}
		System.out.println("END OF DROP\n");
		
		
	}
	
	public Method getMethodFromPath(TreePath path){
		/*		if (path != null) {
		 for(int i=path.getPathCount()-1; i>0; i--){
		 TreeItem ti = (TreeItem) path.getPathComponent(i);
		 if (ti.getData() instanceof Method){
		 return (Method) ti.getData();
		 }
		 }
		 }*/
		return null;
	}
	
	public Entity getEntityFromPath(TreePath path){
		if (path != null) {
			// We skip the first.
			/*	for(int i=path.getPathCount()-2; i>0; i--){
			 TreeItem ti = (TreeItem) path.getPathComponent(i);
			 if (ti.getData() instanceof Entity){
			 return (Entity) ti.getData();
			 }
			 }*/
		}
		return null;
	}
	
	
	public void printNodeContents(TreeNode parent)
	{
		System.out.println("Node is " + parent);
		for(int i = 0 ; i<parent.getChildCount(); i++){
			System.out.println(parent.getChildAt(i));
		}
	}
	public void dropActionChanged(DropTargetDragEvent arg0) {
		System.out.println(arg0);
		
	}
	
	/**
	 * method to give the tree a new analyzemodel if model is updated.
	 * @param model updated model
	 */
	public void updateAnalyzeModel(AnalyzeModel analyzemodel) {
		if (atm == null) return;
		this.analyzemodel = analyzemodel; 
		atm.setAnalyzeModel(analyzemodel);	
	}
	
	public void setStructurePanel(DotPanel dotPanel) {
		this.panel = dotPanel;
	}
	
	
}
