package ucot.ui.gui.tree;

import java.util.HashSet;
import java.util.List;
import java.util.Observable;
import java.util.Observer;
import java.util.Set;
import java.util.Vector;

import javax.swing.event.TreeModelEvent;
import javax.swing.event.TreeModelListener;
import javax.swing.tree.TreePath;

import ucot.model.AnalyzeModel;
import ucot.model.Updation;
import ucot.model.relation.Thing;

public class AnalyzeTreeModel implements
javax.swing.tree.TreeModel, Observer {
	AnalyzeModel model;
	Set<TreeModelListener> listeners = new HashSet<TreeModelListener>();
	TreeItem root;
	
	public void update(Observable o, Object arg) {
		if (! (arg instanceof Updation)) return;
		Updation updation = (Updation) arg;
		Thing thing = updation.getThing();
		
		System.out.println("Updation " +updation.getType() +": "+ thing);
		List<TreeItem> relevants = new Vector<TreeItem>();//root.getRelevants(thing);
		
		List<List<TreeItem>> paths = new Vector<List<TreeItem>>();
		
		for(TreeItem item : relevants){
			List<TreeItem> path = item.getPath();
			if (path != null && path.size()>0){
				paths.add(path);
			}
		}
		
		for(List<TreeItem> path: paths){
			System.out.println("Refreshing path: " + path);
			if (path.size()>0){
				switch(updation.getType()){
				case NEW:
					sendTreeStructureChanged(new TreeModelEvent(this,path.toArray()));
					break;
				case DELETION:
					sendTreeStructureChanged(new TreeModelEvent(this,new Object[]{root}));
					break;
				case MODIFY:
					sendTreeNodesChanged(new TreeModelEvent(this,path.toArray()));
				}
			}
		}
		// Hack. Because we reseive notify after items have been deleted there might
		// not be path to the wanted item. So we need to refresh whole tree.
		if (paths.size()==0){
			sendTreeStructureChanged(new TreeModelEvent(this,new Object[]{root}));	
		}
	}
	
	public AnalyzeModel getAnalyzeModel(){
		return model;
	}
	
	
	public AnalyzeTreeModel(AnalyzeModel am){
		setAnalyzeModel(am);
	}
	
	public void setAnalyzeModel(AnalyzeModel analyzemodel){
		model = analyzemodel;
		root = new EntitiesTreeItem(this, null);
		sendTreeStructureChanged(new TreeModelEvent(this,new Object[]{root}));
		model.addObserver(this);
	}
	
	public void notifyChange(TreeItem item){
		sendTreeStructureChanged(
				new TreeModelEvent(this,new Object[]{item}));
		
	}
	
	public Object getRoot() {
		return root;
	}
	
	public Object getChild(Object parent, int index) {
		if (parent instanceof TreeItem){
			TreeItem ti = (TreeItem) parent;
			List<TreeItem> children = ti.getChildren();
			Object ret = null;
			if (children != null){
				ret = children.get(index);
			}
			return ret;
		} else {
			throw new UnsupportedOperationException();
		}
	}
	
	public int getChildCount(Object parent) {
		if (parent instanceof TreeItem){
			TreeItem ti = (TreeItem) parent;
			List<TreeItem> children = ti.getChildren();
			int ret = 0;
			if (children != null){
				ret = children.size();
			}
			return ret;
		} else {
			throw new UnsupportedOperationException();
		}
	}
	
	public boolean isLeaf(Object node) {
		return getChildCount(node) == 0;
	}
	
	public void valueForPathChanged(TreePath path, Object newValue) {
	}
	
	public int getIndexOfChild(Object parent, Object child) {
		if (parent instanceof TreeItem){
			TreeItem ti = (TreeItem) parent;
			List<TreeItem> children = ti.getChildren();
			int index = -1;
			if (null != children){
				index = children.indexOf(child);
			}
			return index;
		} else {
			throw new UnsupportedOperationException();
		}
	}
	
	public void addTreeModelListener(TreeModelListener l) {
		listeners.add(l);
	}
	
	public void removeTreeModelListener(TreeModelListener l) {
		listeners.remove(l);
		
	}
	
	protected void sendTreeNodesChanged(TreeModelEvent event){
		for(TreeModelListener listener : listeners){
			listener.treeNodesChanged(event);
		}
	}
	
	protected void sendTreeNodesInserted(TreeModelEvent event){
		for(TreeModelListener listener : listeners){
			listener.treeNodesInserted(event);
		}
	}
	protected void sendTreeNodesRemoved(TreeModelEvent event){
		for(TreeModelListener listener : listeners){
			listener.treeNodesRemoved(event);
		}
	}
	
	protected void sendTreeStructureChanged(TreeModelEvent event){
		for(TreeModelListener listener : listeners){
			listener.treeStructureChanged(event);
		}
	}
	
	
}
