package ucot.ui.gui.tree.usecasetree;

import java.util.List;
import java.util.Vector;

import ucot.input.UseCaseCollection;

/**
 * Abstract superclass for all UseCaseTrees treeitems.
 * @author ilanliuk
 *
 */
public abstract class TreeItem {
    
    Object data;
    String name;
    UseCaseTreeModel treemodel;
    TreeItem parent;

    /**
     * Constructor for TreeItem.
     * @param treemodel UseCaseTreeModel
     * @param parent TreeItem wich is parent of this TreeItem.
     */
    public TreeItem(UseCaseTreeModel treemodel, TreeItem parent) {
        this.treemodel = treemodel;
        this.parent = parent;
    }
    
    /**
     * Returns treemodel.
     * @return UseCaseTreeModel
     */
    public UseCaseTreeModel getTreeModel() {
        return this.treemodel;
    }
    
    /**
     * Returns treemodels UseCaseCollection.
     * @return UseCaseCollection
     */
    public UseCaseCollection getUseCaseCollection() {
        return this.treemodel.getUseCaseCollection();
    }

    /**
     * Returns name of this item.
     * @return String name
     */
    public abstract String getName();
 
    /**
     * Return contest of this item in String.
     */
    public String toString() {
        return this.getName();
    }
    
    /**
     * Return children of this TreeItem in a List.
     * @return List of children.
     */
    public List<TreeItem> getChildren() {
        return new Vector<TreeItem>();
    }
    
    /**
     * Returns path to this item as a List of TreeItems.
     * @return List of items in the path.
     */
    public List<TreeItem> getPath() {
        List<TreeItem> path = new Vector<TreeItem>();
        path.add(0, this);
        if (parent != null) {
            path.addAll(0, parent.getPath());
        }
        return path;
    }
}
