package ucot.ui.gui.tree.usecasetree;

import java.awt.Component;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.List;

import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JTree;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.tree.DefaultTreeCellRenderer;
import javax.swing.tree.TreePath;
import javax.swing.tree.TreeSelectionModel;

import ucot.core.ControlInterface;
import ucot.heuristic.HeuristicInterface;
import ucot.input.UseCaseCollection;
import ucot.parser.ParserInterface;
import ucot.ui.UseCasePanelInterface;
import ucot.ui.gui.AddToModelWithDialog;
import ucot.ui.gui.DotPanel;

/**
 * JTree for displaying usecases that are loaded into program.
 * 
 * 
 * TODO: UseCasePanelInterface-rjapinnan ohi mennn.
 * Paikoitellen on kutsuja: ((JPanel)useCasePanel).repaint();
 * Niist pit pst eroon jotenkin.
 *  
 * 
 * @author ilanliuk
 *
 */
public class UseCaseTree extends JTree {
    
    private TreeSelectionListener treeSelectionListener;
    private MouseListener mouseListener;
    
    private UseCaseTreeModel useCaseTreeModel;
    private DotPanel dotPanel = null;
    private ControlInterface controlInterface = null;
    
    private UseCasePanelInterface useCasePanel;
    
    // fonts that are used in drawing UseCaseTreeItems
    // highlightFont is for items that are already in 
    // analyzemodel.
    private Font defaultFont = new Font("Dialog", Font.PLAIN, 12);
    private Font highlightFont = new Font("Dialog", Font.ITALIC, 12);
    
    
    
    public UseCaseTree(UseCaseCollection c, ControlInterface core) {
        super();
        this.controlInterface = core;
        this.useCaseTreeModel = new UseCaseTreeModel(c);
        this.getSelectionModel().setSelectionMode(TreeSelectionModel.SINGLE_TREE_SELECTION);
        
        this.setModel(this.useCaseTreeModel);
        
        // Create custom CellRenderer so we can highlight the items
        // we want.
        this.setCellRenderer( new DefaultTreeCellRenderer() {
            
            @Override
            public Component getTreeCellRendererComponent(JTree tree, Object value, boolean sel, boolean expanded, boolean leaf, int row, boolean hasFocus) {
                // TODO Auto-generated method stub
                super.getTreeCellRendererComponent(tree, value, sel, expanded, leaf,
                        row, hasFocus);
                
                setFont(defaultFont);
                
                if (value instanceof UseCaseTreeItem) {
                    UseCaseTreeItem item = (UseCaseTreeItem) value;
                    
                    if (item.isUsecaseInAnalyzemodel()) {
                        setFont(highlightFont);
                    } 
                }
                return this;
            }   
        } ); 
        
        //
        // Create TreeSelectionListener -->
        //
        // if UseCaseTreeItem -node is selected from tree
        // - highlight entities and relations in DotPanel
        //   from that usecase 
        // - show usecase steps in UsecasePanel
        //
        this.treeSelectionListener = new TreeSelectionListener() {
            
            public void valueChanged(TreeSelectionEvent e) {
                
                TreeItem ti = (TreeItem) e.getPath().getLastPathComponent();
                
                if (ti instanceof UseCaseTreeItem) {
                    
                    UseCaseTreeItem c = (UseCaseTreeItem) ti;
                    
                    if (c.isUsecaseInAnalyzemodel()) {
                        System.out.println(
                                "UseCaseTree: highlight entities from usecase: " + c.getName());
                        if ( dotPanel != null ) {
                            dotPanel.highlight(c.getUseCase().getAnalyzeModel());
                        }
                    } else {
                        dotPanel.highlight(null);
                    }
                    // Show selected usecase in useCasePanel
                    useCasePanel.showUseCase(c.getUseCase());
                } else {
                    useCasePanel.Clear();
                }
            }
            
        };
        //
        // --> TreeSelectionListener
        //
        this.addTreeSelectionListener(this.treeSelectionListener);
        
        
        //
        // Create MouseListener -->
        //
        this.mouseListener = new MouseListener() {
            
            public void mouseExited(MouseEvent e) {
                // nothing to do here...
            }
            
            public void mouseEntered(MouseEvent e) {
                // nothing to do here...
            }
            
            public void mouseReleased(MouseEvent e) {
                // nothing to do here...
            }
            
            public void mousePressed(MouseEvent e) {
                // nothing to do here...
            }
            
            public void mouseClicked(MouseEvent e) {
                
                final TreePath selectionPath = getPathForLocation(e.getX(), e.getY());
                final ControlInterface core = controlInterface;
                
                // if nothing was selected, just exit
                if (selectionPath == null) {
                    return;
                }
                
                // set selection correct
                setSelectionPath(selectionPath);
                
                // if doubleclicked with first mousebutton
                // on UseCaseTreeItem add UseCase to model
                if (e.getButton() == MouseEvent.BUTTON1
                        && e.getClickCount() == 2) {
                    
                    Object o = selectionPath.getLastPathComponent();
                    
                    if (o instanceof UseCaseTreeItem) {
                        UseCaseTreeItem item = (UseCaseTreeItem) o;
                        
                        // if usecase is already in model
                        // nothing to do.
                        if (item.isUsecaseInAnalyzemodel()) {
                            return;
                        }
                        
                        // try to add usecase into model
                        try {
                            core.addToAnalyzeModel(item.getUseCase());
                        } catch (Exception ex) {
                            System.err.println("usecasetree> Mouselistener> can't add usecase into analyzemodel.");
                            ex.printStackTrace();
                        }
                        repaint();
                        ((JPanel)useCasePanel).repaint();                        
                    }
                }    
                
                // if user clicked second mouse button
                // (MouseEvent.BUTTON2 is scroll)
                if (e.getButton() == MouseEvent.BUTTON3) {
                    
                    // if selection was not TreeItem just exit..
                    final Object o = selectionPath.getLastPathComponent();
                    if (!(o instanceof ucot.ui.gui.tree.usecasetree.TreeItem)) {
                        return;
                    }
                    
                    // create popupmenu and options for it.
                    JPopupMenu popup = new JPopupMenu();
                    
                    //
                    // if selected is a UseCaseTreeItem
                    // add option to menu to add it to model
                    // with default parser and heuristics and
                    // with selected parser and heuristics
                    //
                    if (o instanceof UseCaseTreeItem) {
                        
                        JMenuItem addSingle = new JMenuItem("Add to model with defaults");
                        // create actionlistener for addSingle-menuitem.
                        addSingle.addActionListener(new ActionListener() {
                            public void actionPerformed(ActionEvent e) {
                                if (o instanceof UseCaseTreeItem) {
                                    
                                    UseCaseTreeItem item = (UseCaseTreeItem) o;
                                    if (item.isUsecaseInAnalyzemodel())
                                        return;
                                    try {
                                        core.addToAnalyzeModel(item.getUseCase());
                                    } catch (Exception ex) {
                                        System.err.println("Error when trying to add \"" + item.getUseCase().getName() + "\" into analyzemodel.");
                                        ex.printStackTrace();
                                    }
                                }
                                repaint();
                                ((JPanel)useCasePanel).repaint();
                                
                            }
                        });
                        
                        // create menu for launching dialog for selecting 
                        // parser and heuristic to use
                        JMenuItem selectAddSingle = new JMenuItem("Add to model with...");
                        
                        selectAddSingle.addActionListener(new ActionListener() {
                            public void actionPerformed(ActionEvent e) {
                                if (o instanceof UseCaseTreeItem) {

                                    // if showDialog returns false, user clicked Cancel
                                    // in dialog and we do nothing
                                    if (!AddToModelWithDialog.showDialog(core)) {
                                        return;
                                    }
                                    
                                    ParserInterface parser = AddToModelWithDialog.getParser();
                                    HeuristicInterface heuristic = AddToModelWithDialog.getHeuristic();                                            
                                    
                                    UseCaseTreeItem item = (UseCaseTreeItem) o;
                                    // if usecase is already in model there is no reason 
                                    // to add it again
                                    if (item.isUsecaseInAnalyzemodel()) {
                                        return;
                                    }
                                    try {
                                        core.addToAnalyzeModel(item.getUseCase(), parser, heuristic);
                                    } catch (Exception ex) {
                                        System.err.println("Error when trying to add \"" + item.getUseCase().getName() + "\" into analyzemodel.");
                                        ex.printStackTrace();
                                    }
                                }
                                repaint();
                                ((JPanel)useCasePanel).repaint();
                            }
                        });
                        
                        popup.add(addSingle);
                        popup.add(selectAddSingle);
                        
                        
                        //
                        // if UseCaseTreeItem has children, add also option 
                        // to add whole subtree to model.
                        //
                        if (!getModel().isLeaf(o)) {
                            
                            JMenuItem addRecursively = new JMenuItem("Add subtree to model with defaults");
                            // create actionlistener for addRecursively
                            // -menuitem.
                            addRecursively.addActionListener(new ActionListener() {
                                public void actionPerformed(ActionEvent e) {
                                    if (o instanceof UseCaseTreeItem) {
                                        UseCaseTreeItem item = (UseCaseTreeItem) o;
                                        addSubtreeToModel(item);
                                    }
                                    repaint();
                                    ((JPanel)useCasePanel).repaint();
                                }
                            });
                            
                            // create submenu for selecting parser
                            // to use
                            JMenuItem selectAddRecursively = new JMenuItem("Add subtree to model with...");
                            selectAddRecursively.addActionListener(new ActionListener() {
                                public void actionPerformed(ActionEvent e) {
                                    if (o instanceof UseCaseTreeItem) {

                                        // if showDialog returns false, user clicked Cancel
                                        // in dialog and we do nothing
                                        if (!AddToModelWithDialog.showDialog(core)) {
                                            return;
                                        }
                                        
                                        ParserInterface parser = AddToModelWithDialog.getParser();
                                        HeuristicInterface heuristic = AddToModelWithDialog.getHeuristic();                                            
                                       
                                        UseCaseTreeItem item = (UseCaseTreeItem) o;
                                        addSubtreeToModel(item, parser, heuristic);
                                    }
                                    repaint();
                                    ((JPanel)useCasePanel).repaint();
                                }
                            });
                            popup.addSeparator();
                            popup.add(addRecursively);
                            popup.add(selectAddRecursively);
                        }
                        
                    }
                    
                    
                    //
                    // if selected node is FileTreeItem add option to menu
                    // to add all UseCases from file to model
                    //
                    if (o instanceof FileTreeItem) {
                        JMenuItem addFile = new JMenuItem("Add file to model with defaults");
                        // create actionlistener for addFile -menuitem
                        addFile.addActionListener( new ActionListener() {
                            public void actionPerformed(ActionEvent e) {
                                
                                // if FileTreeItem is selected, add all
                                // usecases from that file to analyzemodel.
                                if (o instanceof FileTreeItem) {
                                    addSubtreeToModel((FileTreeItem) o);
                                }
                                repaint();
                                ((JPanel)useCasePanel).repaint();
                            }
                            
                        });
                        JMenuItem addFileWith = new JMenuItem("Add file to model with..");
                        addFileWith.addActionListener( new ActionListener() {
                            public void actionPerformed(ActionEvent e) {
                                if (o instanceof FileTreeItem) {

                                    // if showDialog returns false, user clicked Cancel
                                    // in dialog and we do nothing
                                    if (!AddToModelWithDialog.showDialog(core)) {
                                        return;
                                    }
                                    
                                    ParserInterface parser = AddToModelWithDialog.getParser();
                                    HeuristicInterface heuristic = AddToModelWithDialog.getHeuristic();                                            

                                    addSubtreeToModel((FileTreeItem) o, parser, heuristic);
                                }
                                repaint();
                            }
                            
                        });
                        popup.add(addFile);
                        popup.add(addFileWith);
                    }
                    
                    if (o instanceof FilesTreeItem) {
                        JMenuItem addFiles = new JMenuItem("Add files to model with defaults");
                        addFiles.addActionListener(new ActionListener() {
                            
                            public void actionPerformed(ActionEvent e) {
                                if (o instanceof FilesTreeItem) {
                                    addSubtreeToModel((FilesTreeItem) o);
                                }
                                repaint();
                                ((JPanel)useCasePanel).repaint();
                            }
                        });
                        
                        JMenuItem addFilesWith = new JMenuItem("Add files to model with..");
                        addFilesWith.addActionListener( new ActionListener() {
                        
                            public void actionPerformed(ActionEvent e) {
                                if (o instanceof FilesTreeItem) {

                                    // if showDialog returns false, user clicked Cancel
                                    // in dialog and we do nothing
                                    if (!AddToModelWithDialog.showDialog(core)) {
                                        return;
                                    }
                                    
                                    ParserInterface parser = AddToModelWithDialog.getParser();
                                    HeuristicInterface heuristic = AddToModelWithDialog.getHeuristic();                                            

                                    addSubtreeToModel((FilesTreeItem) o, parser, heuristic);
                                }
                            }
                        
                        } );
                        popup.add(addFiles);
                        popup.add(addFilesWith);
                    }
                    
                    // Show popup menu relative to the components
                    // coordinates.
                    // This shows popup exactly where user pressed.
                    popup.show(e.getComponent(), e.getX(), e.getY());
                }    
            }
        };
        //
        // --> MouseListener
        //
        
        this.addMouseListener(this.mouseListener);
    } 
    
    /**
     * Method adds UseCase pointed at UseCaseTreeItem and it's subUsecases into model
     * with given parser and heuristics. Skips UseCases that have already been added 
     * to model.
     * @param item UseCaseTreeItem to add to model.
     * @param parser ParserInterface
     * @param heuristic HeuristicInteface 
     */
    private void addSubtreeToModel(UseCaseTreeItem item, ParserInterface parser, HeuristicInterface heuristic) {
        
        // add subusecases to recursively model 
        List<UseCaseTreeItem> list = item.getChildren();
        for (UseCaseTreeItem child : list) {
            addSubtreeToModel(child, parser, heuristic);
        }
        
        // if usecase is already in model
        // there is no reason to add it again
        if (item.getUseCase().isUseCaseAnalyzed())
            return;
        
        // try to add usecase to model
        try {
            controlInterface.addToAnalyzeModel(item.getUseCase(), parser, heuristic);
        } catch (Exception ex) {
            System.err.println("Error when trying to add \"" + item.getUseCase().getName() + "\" into analyzemodel.");
            ex.printStackTrace();
        }
    }

    /**
     * Method adds UseCase pointed at UseCaseTreeItem and it's subUsecases into model
     * with default parser and heuristics. Skips UseCases that have already been added 
     * to model.
     * @param item UseCaseTreeItem to add to model.
     */
    private void addSubtreeToModel(UseCaseTreeItem item) {
        addSubtreeToModel(item, controlInterface.getCurrentParser(), controlInterface.getCurrentHeuristic());
    }
    
    /**
     * Method to add all UseCases from file pointed out at FileTreeItem with given 
     * parser and heuristics.
     * @param item FileTreeItem
     * @param parser ParserInterface
     * @param heuristic HeuristicInteface 
     */
    private void addSubtreeToModel(FileTreeItem item, ParserInterface parser, HeuristicInterface heuristic) {
        
        // get usecases from this file
        List<UseCaseTreeItem> list = item.getChildren();
        
        // add usecases to model
        for (UseCaseTreeItem child : list) {
            addSubtreeToModel(child, parser, heuristic);
        }
    }
    /**
     * Method to add all UseCases from file pointed out at FileTreeItem with default 
     * parser and heuristics.
     * @param item FileTreeItem
     */    
    private void addSubtreeToModel(FileTreeItem item) {
        addSubtreeToModel(item, controlInterface.getCurrentParser(), controlInterface.getCurrentHeuristic());
    }

    /**
     * Method to add all usecase-files that are loaded into program into 
     * model with given parser and heuristics.
     * @param item FilesTreeItem
     * @param parser ParserInterface
     * @param heuristic HeuristicInteface 
     */
    private void addSubtreeToModel(FilesTreeItem item, ParserInterface parser, HeuristicInterface heuristic) {
        
        // get all loaded usecase-files
        List<FileTreeItem> list = item.getChildren();
        
        // add files to model
        for (FileTreeItem file : list) {
            addSubtreeToModel(file, parser, heuristic);
        }
    }
    
    /**
     * Method to add all usecase-files that are loaded into program into 
     * model with given parser and heuristics.
     * @param item FilesTreeItem
     */
    private void addSubtreeToModel(FilesTreeItem item) {
        addSubtreeToModel(item, controlInterface.getCurrentParser(), controlInterface.getCurrentHeuristic());
    }
    
    public void setDotPanel(DotPanel panel) {
        dotPanel = panel;
    }
    
    public void setUseCasePanel(UseCasePanelInterface useCasePanel) {
        this.useCasePanel = useCasePanel;
    }
    
}
