package ucot.ui.gui.tree.usecasetree;

import java.util.HashSet;
import java.util.List;
import java.util.Observable;
import java.util.Observer;
import java.util.Set;

import javax.swing.event.TreeModelEvent;
import javax.swing.event.TreeModelListener;
import javax.swing.tree.TreeModel;
import javax.swing.tree.TreePath;

import ucot.input.UseCaseCollection;

public class UseCaseTreeModel implements TreeModel, Observer {

    protected Set<TreeModelListener> listeners = new HashSet<TreeModelListener>();
    private UseCaseCollection collection;
    private TreeItem root;
    
    public void update(Observable o, Object arg) {
        for (TreeModelListener l : listeners) {
            l.treeStructureChanged(new TreeModelEvent(this, new Object[]{root}));
        }
    }
    
    public UseCaseTreeModel(UseCaseCollection c) {
        super();
        this.collection = c;
        this.root = new FilesTreeItem(this, null);
        c.addObserver(this);
    }

    public Object getRoot() {        
        return this.root;
    }

    public Object getChild(Object parent, int index) {
        if (parent instanceof TreeItem) {
            TreeItem ti = (TreeItem) parent;
            List<TreeItem> children = ti.getChildren();
            TreeItem result = null;
            if (children != null) {
                result = children.get(index);
            }
            return result;
        } else throw new UnsupportedOperationException("parent is not instance of TreeItem");        
    }

    public int getChildCount(Object parent) {
        if (parent instanceof TreeItem) {
            TreeItem ti = (TreeItem) parent;
            List<TreeItem> children = ti.getChildren();
            int result = 0;
            if (children != null) {
                result = children.size();
            }
            return result;
        } else throw new UnsupportedOperationException("parent is not instance of TreeItem");        
    }

    public boolean isLeaf(Object node) {
        return getChildCount(node) == 0;
    }

    public void valueForPathChanged(TreePath path, Object newValue) {
    }

    public int getIndexOfChild(Object parent, Object child) {
        if (parent instanceof TreeItem) {
            TreeItem ti = (TreeItem) parent;
            List<TreeItem> children = ti.getChildren();
            int index = -1;
            if (children != null) {
                index = children.indexOf(child);
            }
            return index;
        } else throw new UnsupportedOperationException("parent is not instance of TreeItem");        
    }

    public void addTreeModelListener(TreeModelListener l) {
        listeners.add(l);
    }

    public void removeTreeModelListener(TreeModelListener l) {
        listeners.remove(l);
    }

    public UseCaseCollection getUseCaseCollection() {

        return this.collection;
    }
    
}
